;; Modus for Emacs-kurs.

;; Last inn denne filen med `M-x load-file <RET>' og si `M-x
;; kurs-mode' i tekst.txt.

(defface kurs-overskrift-face
  `((((type tty) (class color))
     (:foreground "blue"))
    (((type tty) (class mono))
     (:inverse-video t))
    (((class color) (background dark))
     (:foreground "lightblue" :weight bold :height 200))
    (((class color) (background light))
     (:foreground "darkblue" :weight bold :height 200))
    (t (:foreground "blue" :weight bold :height 200)))
  "Kurs-overskrift")

(defvar kurs-mode-map nil)
(setq kurs-mode-map (make-sparse-keymap))
(define-key kurs-mode-map " " 'kurs-neste-side)
(define-key kurs-mode-map "n" 'kurs-neste-side)
(define-key kurs-mode-map "p" 'kurs-forrige-side)
(define-key kurs-mode-map "\M-<" 'kurs-topp)
(define-key kurs-mode-map "\M->" 'kurs-bunn)
(define-key kurs-mode-map "\M-?" 'kurs-info)

(defun kurs-neste-side (&optional arg)
  "Gå til neste side i kurset.
Med prefiksargument N, gå N sider forover."
  (interactive "p")
  (when (> (point) (point-min))
    ;; avoid moving two pages when point is at end of page
    (backward-char))
  (widen)
  (forward-page arg)
  (narrow-to-page))

(defun kurs-forrige-side (&optional arg)
  "Gå til forrige side i kurset.
Med prefiksargument N, gå N sider bakover."
  (interactive "p")
  (widen)
  (backward-page)
  (backward-page arg)
  (narrow-to-page))

(defun kurs-topp ()
  "Gå til første side."
  (interactive)
  (widen)
  (goto-char (point-min))
  (narrow-to-page))

(defun kurs-bunn ()
  "Gå til siste side."
  (interactive)
  (widen)
  (goto-char (point-max))
  (narrow-to-page))

(defun kurs-sidenr ()
  "Returnerer nåværende sidenummer."
  (save-restriction
    (widen)
    (save-excursion
      (let ((sidenr 1)
	    (pt (point)))
	(goto-char 1)
	(while (re-search-forward page-delimiter pt t)
	  (setq sidenr (1+ sidenr)))
	sidenr))))

(defun kurs-antall-sider ()
  "Returnerer antall sider i bufferet."
  (save-restriction
    (widen)
    (save-excursion
      (goto-char (point-max))
      (kurs-sidenr))))

(defun kurs-info ()
  "Vis nåværende sidenummer og antall sider."
  (interactive)
  (message "Side %d av %d" (kurs-sidenr) (kurs-antall-sider)))

(setq kurs-mode-font-lock-keywords
 '(("^((.*))$" . 'kurs-overskrift-face)))

(define-derived-mode kurs-mode fundamental-mode "kurs"
  "Durmodus for Emacs-kurs.

Begrenser vist tekst til én side av gangen.  Bruk `n' og `p' for
å skifte side.  For å se alle sidene, si `C-x n w' (widen).  Bruk
`M-?' for å finne ut hvor i filen du er."
  (setq mode-name "kurs")
  (setq font-lock-defaults '(kurs-mode-font-lock-keywords))
  (narrow-to-page))

(provide 'kurs-mode)
